#DECtalk Express Driver for NVDA c2012 Alex H. <linuxx64.bashsh@gmail.com> and Tyler Spivey <tspivey@pcdesk.net>
#Change this to the port the synth is on
PORT="com3"
import threading
import Queue
import synthDriverHandler
from synthDriverHandler import SynthDriver
import speech
import serial
import re
import logging
lastindex = 0
minRate = 75
maxRate = 650
minPitch = 50
maxPitch = 350
minInflection=0
maxInflection=250
chars = []
index_re = re.compile(r'\[:index (\d+)\]')
#This can't terminate...
class IndexingThread(threading.Thread):
 def run(self):
  chars = []
  while True:
   byte = port.read(1)
   global lastindex
   chars.append(byte)
   if chars[-1] == ']':
    s = "".join(chars)
    m = index_re.search(s)
    chars = []
    if m:
     lastindex = int(m.group(1))

def clear_queue(q):
 try:
  while True:
   q.get_nowait()
 except:
  pass

port = None
serialQueue = Queue.Queue()
class BgThread(threading.Thread):
 def __init__(self, q):
  threading.Thread.__init__(self)
  self.setDaemon(True)
  self.q = q

 def run(self):
  try:
   while True:
    func, args, kwargs = self.q.get()
    if not func:
     break
    func(*args, **kwargs)
    self.q.task_done()
  except:
   logging.error("bgThread.run", exc_info=True)

def _bgExec(q, func, *args, **kwargs):
 q.put((func, args, kwargs))

def bgWrite(text):
 #timeout errors. Ignore all errors for now, fix later to ignore the specific error
 try:
  port.write(text)
 except:
  pass


class SynthDriver(SynthDriver):
 name = "express"
 description = "DECtalk Express"
 supportedSettings = (SynthDriver.RateSetting(),SynthDriver.PitchSetting(),SynthDriver.InflectionSetting())

 @classmethod
 def check(cls):
  bgWrite("@")
  return True

 def __init__(self):
  global port
  port = serial.serial_for_url(PORT, 9600)
  port.setDsrDtr(0)
  self.serialThread = BgThread(serialQueue)
  self.serialThread.start()
  self.indexingThread = IndexingThread()
  self.indexingThread.start()
  self.dt_rate = 200
  self.dt_pitch = 100
  self.dt_inflection = 100

 def speak(self, speechSequence):
  text_list = []
  for item in speechSequence:
   if isinstance(item, basestring):
    text_list.append(item)
   elif isinstance(item,speech.IndexCommand):
    text_list.append("[:i r %d]" % item.index)
  #No unicode here. Do something better with this later
  text = u" ".join(text_list).encode('ascii', 'replace')+'\13'
  text = "[:ra %s :dv ap %s :dv pr %s] %s" % (self.dt_rate, self.dt_pitch, self.dt_inflection, text)

  if text:

   _bgExec(serialQueue, bgWrite, text)

 def cancel(self):
  clear_queue(serialQueue)
  try:
   port.write('\x03\x15') #Always resume so if ctrl is hit after pausing, it's always un-paused at next text event.
  except serial.SerialTimeoutException:
   pass

 def _set_rate(self, vl):
  self.dt_rate = self._percentToParam(vl,minRate,maxRate)

 def _get_rate(self):
  return self._paramToPercent(self.dt_rate, minRate, maxRate)

 def _set_pitch(self, vl):
  self.dt_pitch = self._percentToParam(vl, minPitch, maxPitch)

 def _get_pitch(self):
  return self._paramToPercent(self.dt_pitch, minPitch, maxPitch)

 def _set_inflection(self, vl):
  self.dt_inflection = self._percentToParam(vl,minInflection,maxInflection)

 def _get_inflection(self):
  return self._paramToPercent(self.dt_inflection, minInflection, maxInflection)

 def terminate(self):
  serialQueue.put((None, None, None))
  port.close()

 def _get_lastIndex(self):
  return lastindex

 def pause(self,switch):
  if switch:
   bgWrite('^n')
  else:
   bgWrite('^o')
